function [U,D,devs,conts,vmin,vmax]=gamepreprocess(game,vmini,delta);
%Preprocessing game. Only for 2 player games!
%Inputs: 
%game = multidimensional table of players' stage game payoffs 
%game(i,j,k)=plyer k's payoff when player 1 takes action i and player two takes action j
%vmini 2 x 1 vector where vmini(i) the smallest payoff for player i (can be left empty)
%delta = vector of discount factors, delta(i) = player i's discount factor  
%Outputs:
%U utilities in a table U(i,k) player i's stage game payoff corresponding to k'th action profile
%D is a table corresponding to U where the payoffs are the RHS values of IC conditions
%devs=players deviation payoffs (dimensions as in U)
%conts=players required continuation payoffs (dimensions as in U)
%vmin = smallest payoffs (equal to vmini if vmini is given)
%vmax = largest stage game payoffs

if length(delta)==1
delta=delta.*[1 1];
end

[n m k]=size(game);
U=zeros(2,n*m);
D=zeros(2,n*m);
devs=U;
conts=devs;
if isempty(vmini)
vmin=zeros(2,1);
vmin(1)=min(max(game(:,:,1),[],1),[],2);
vmin(2)=min(max(game(:,:,2),[],2),[],1);
else
vmin=vmini;
end
vmax=vmin;
vmax(1)=max(max(game(:,:,1),[],1),[],2);
vmax(2)=max(max(game(:,:,2),[],2),[],1);
Utrial=zeros(2,1); Dtrial=Utrial;
counter=0;
for i=1:n
for j=1:m
dev1=max(max(game(:,j,1),[],1),vmin(1));
dev2=max(max(game(i,:,2),[],2),vmin(2));
Utrial(1)=game(i,j,1);
Utrial(2)=game(i,j,2);
Dtrial(1)=(1-delta(1))*dev1+delta(1)*vmin(1);
Dtrial(2)=(1-delta(2))*dev2+delta(2)*vmin(2);
contrial1=(Dtrial(1)-(1-delta(1))*Utrial(1))/delta(1);
contrial2=(Dtrial(2)-(1-delta(2))*Utrial(2))/delta(2);
if contrial1<=vmax(1) && contrial2<=vmax(2)
%otherwise action cannot be played, hence do not take into account
counter=counter+1;
devs(:,counter)=[dev1;dev2];
U(:,counter)=Utrial(:);
D(:,counter)=Dtrial(:);
conts(:,counter)=[contrial1;contrial2];
conts(1,counter)=max(conts(1,counter),vmin(1));
conts(2,counter)=max(conts(2,counter),vmin(2));
D(1,counter)=max(D(1,counter),vmin(1));
D(2,counter)=max(D(2,counter),vmin(2));
end

end
end
U=U(:,1:counter);
D=D(:,1:counter);
devs=devs(:,1:counter);
conts=conts(:,1:counter);
