function outvertices=grahamscan(vertices,Tol)
%Function for finding the relevant vertices of a convex hull of a set of two dimensional vectors.
%Vertices is a table of 2-dimensional vectors where each column corersponds to one vector.
%Tol is the numerical accuracy.
%The algorithm is based on Graham Scan but is slightly tuned for better efficiency.

cntr=1;
vertstosearch=vertices;
[mval1,ind]=max(vertstosearch(1,:));
indices=find(vertstosearch(1,:)==mval1);
vertstosearch=vertstosearch(:,indices);
[mval2,ind]=max(vertstosearch(2,:));
verts(:,cntr)=vertstosearch(:,ind);

%Loop "quadrant" by "quadrant", therefore 4 consequtive loops
%first "quadrant" dimension 1 decreases and dimension 2 increases
%second "quadrant" dimension 1 decreases and dimension 2 decreases
%third "quadrant" dimension 1 increases and dimension 2 decreases
%fourth "quadrant" dimension 1 increases and dimension 2 increases

stopflag=false;
while stopflag==false
indices=find(vertices(2,:)>=mval2 & vertices(1,:)<mval1);
if ~isempty(indices)
cntr=cntr+1;
vertstosearch=vertices(:,indices);
[mval1,ind]=max(vertstosearch(1,:));
indices=find(vertstosearch(1,:)==mval1);
vertstosearch=vertstosearch(:,indices);
[mval2,ind]=max(vertstosearch(2,:));
verts=remover(vertstosearch(:,ind),verts,1);
else
stopflag=true;
end
end
%No we have one qudrant, then the next one, now mval1 and mval2 decrease
outvertices=verts;
verts=outvertices(:,end);

stopflag=false;
while stopflag==false
indices=find(vertices(2,:)<mval2 & vertices(1,:)<=mval1);
if ~isempty(indices)
cntr=cntr+1;
vertstosearch=vertices(:,indices);
[mval2,ind]=max(vertstosearch(2,:));
indices=find(vertstosearch(2,:)==mval2);
vertstosearch=vertstosearch(:,indices);
[mval1,ind]=max(vertstosearch(1,:));
verts=remover(vertstosearch(:,ind),verts,1);
else
stopflag=true;
end
end
outvertices=[outvertices verts(:,2:end)];
verts=outvertices(:,end);
%No we have two qudrants, then the next one, now mval1 increases and mval2 decreases

stopflag=false;
while stopflag==false
indices=find(vertices(2,:)<=mval2 & vertices(1,:)>mval1);
if ~isempty(indices)
cntr=cntr+1;
vertstosearch=vertices(:,indices);
[mval1,ind]=min(vertstosearch(1,:));
indices=find(vertstosearch(1,:)==mval1);
vertstosearch=vertstosearch(:,indices);
[mval2,ind]=min(vertstosearch(2,:));
verts=remover(vertstosearch(:,ind),verts,2);
else
stopflag=true;
end
end
outvertices=[outvertices verts(:,2:end)];
verts=outvertices(:,end);

stopflag=false;
while stopflag==false
indices=find(vertices(2,:)>=mval2 & vertices(1,:)>mval1);
if ~isempty(indices)
cntr=cntr+1;
vertstosearch=vertices(:,indices);
[mval2,ind]=min(vertstosearch(2,:));
indices=find(vertstosearch(2,:)==mval2);
vertstosearch=vertstosearch(:,indices);
[mval1,ind]=min(vertstosearch(1,:));
verts=remover(vertstosearch(:,ind),verts,2);
else
stopflag=true;
end
end
outvertices=[outvertices verts(:,2:end)];

if norm(outvertices(:,end)-outvertices(:,1))<Tol
outvertices=outvertices(:,1:end-1);
end
endfunction
%----------------------------------------------------------------
function verts=remover(trial,vertices,casef)
%Auxiliary function used in grahamscan.
%Input: trial vertex, set of vertices, casef flag on the orientation
stopflag=false;
verts=vertices;
n=zeros(2,1);

if size(vertices,2)>=2
while stopflag==false
v1=verts(:,end)-verts(:,end-1);
if v1(2)!=0
n(1)=1;n(2)=-v1(1)/v1(2);
else
n(2)=1;n(1)=-v1(2)/v1(1);
end

if casef==1 && n(2)<0
n=-n;
elseif casef==2 && n(2)>0
n=-n;
end

if casef==1 && n(2)==0 && n(1)>0
n=-n;
elseif casef==2 && n(2)==0 && n(1)<0
n=-n;
end


v2=trial-verts(:,end);

if n'*v2>=0
verts=verts(:,1:end-1);
else
stopflag=true;
end

if size(verts,2)==1
stopflag=true;
end

end
verts=[verts trial];
else
verts=[vertices trial];
end
endfunction
