function [vout,Lout]=minpayoffs(A,L,U,delta)
%Computing the smallest payoffs that can follow all the action profiles.
%Inputs:
%A = adjacency matrix of the graph
%L = labels of the action profiles corresponding to the nodes of the graph
%U = stage game payoffs
%delta = discount factors
%Outputs:
%vout table of the smallest equilibrium payoffs after the action profiles
%vout(i,j) payoff of player i corresponding to j'th action profile
%Lout labels of action profiles L(j) is the label, i.e., number corresponding to the element of U that is played in j'th action profile

n=length(A);
states=1:1:n;
actions=states;
f1=-Inf.*ones(n,n);
f2=f1;
U=-[0 U(1,:); 0 U(2,:)];

g1=repmat(states,[n 1]);
g2=repmat(states,[n 1]);

for i=1:length(states)
successors=find(A(i,:)>0);
f1(i,successors)=U(1,L(successors)+1);
f2(i,successors)=U(2,L(successors)+1);
end

%Note dpsolver solves maximization problems therefore - in front of U
[v1,x,pstar] = dpsolver(f1,g1,delta(1));
[v2,x,pstar] = dpsolver(f2,g2,delta(2));

v=[v1'.*(1-delta(1)); v2'.*(1-delta(2))];%normalize the payoffs
%states of interest
actionnodes=find(A(1,:)>0);
vout=-v(:,actionnodes);
Lout=L(actionnodes);
