function [E,vout,nodesds,vouds]=postprocess(nodeset,U,vmin,L,delta,base)
%Post-processing the output into an elementary set.
%Inputs:
%nodeset = subpaths of interest
%U = stage game payoffs
%vmin = table of smallest payoffs that can follow the action profiles
%L = labels of actions corresponding to vmin 
%delta = discount factors
%base = base number for storing sequences of action profiles
%Outputs 
%E is the updated elementary set (a cell-array), and vout a cell-array 
%of corresponding smallest payoffs
%For diagnostic purposes
%nodesds list of all subpaths in the processed elementary set
%vouds the smallest payoffs that can follow the subpaths in nodesds

E{1}=nodeset(find(floor(nodeset./base)==0));
vout{1}=ones(2,length(E{1}));

for i=1:length(E{1})
ind=find(E{1}(i)==L);
vout{1}(1,:)=vout{1}(1,:).*vmin(1,ind);
vout{1}(2,:)=vout{1}(2,:).*vmin(2,ind);
end
vouds=vout{1};
nodesds=E{1};

for i=1:length(nodeset)
trial=nodeset(i);

m=length(num2str(trial));
x=log(base)/log(10);
n=ceil(m/x);

if n>1 %one length elementary paths are already known

actions=getactions(trial,base,n);%an array of indices of actions played along trial

for i=2:n
%Testing condition 3
ind=find(actions(i)==L);
vtrial=[(1-delta(1))*delta(1).^[0:i-2]*U(1,actions(2:i))'+delta(1)^(i-1).*vmin(1,ind);(1-delta(2))*delta(2).^[0:i-2]*U(2,actions(2:i))'+delta(2)^(i-1).*vmin(2,ind)];
ind=find(actions(1)==L);
flag1=(vtrial(1)>=vmin(1,ind));
flag2=(vtrial(2)>=vmin(2,ind));
if flag1 && flag2
if length(E)>=i
[E{i},indx]=unique([E{i} actions(1:i)*(flip(base.^[0:i-1]'))]);
vaux=[vout{i} vtrial];
vout{i}=vaux(:,indx);
vouds=[vouds vtrial];
nodesds=[nodesds trial];
else
E{i}=actions(1:i)*(flip(base.^[0:i-1]'));
vout{i}=vtrial;
vouds=[vouds vtrial];
nodesds=[nodesds trial];
end
break
end

if i==n %in this case the trial itself is elementary
if length(E)>=i
[E{i},indx]=unique([E{i} trial]);
vaux=[vout{i} vtrial];
vout{i}=vaux(:,indx);
vouds=[vouds vtrial];
nodesds=[nodesds trial];
else
E{i}=trial;
vout{i}=vtrial;
vouds=[vouds vtrial];
nodesds=[nodesds trial];
end

end

end%end of for
end %end of if
end%end of for
endfunction

%-----------------------------------------------------------------------------
function actions=getactions(trial,base,n)
%Actions corresponding to trial
actions=flip(floor(base.*(trial./base.^[1:n]-floor(trial./base.^[1:n]))+1/base^(n+1)));
%Warning: flooring may cause errors, so therefore add 1/base^(n+1)
endfunction
